@@ -35,6 +35,9 @@
   *                                                       buttons are shown no
   *                                                       matter what (OPTIONAL;
   *                                                       default = FALSE).
+  *
+  * @return string HTML for the buttons to be displayed (for use in 1.4.x)
+  *
   */
 function get_spam_buttons(&$buttons, 
                           $dont_show_if_report_by_move_to_folder=FALSE)
@@ -51,7 +54,8 @@ function get_spam_buttons(&$buttons,
           $sb_show_spam_button_folder,
           $sb_show_spam_button_folder_allow_override,
           $sb_show_not_spam_button_folder,
-          $sb_show_not_spam_button_folder_allow_override;
+          $sb_show_not_spam_button_folder_allow_override,
+          $extra_buttons;
  
    $passed_ent_id = 0;
    sqgetGlobalVar('passed_ent_id', $passed_ent_id, SQ_FORM);
@@ -89,36 +93,36 @@ function get_spam_buttons(&$buttons,
    // these options used to be a strings; these lines convert
    // values that may have already been stored in user prefs
    // as strings into proper array format (also unpacks
-   // serialized arrays for SM v1.5.2+)...  perhaps this
+   // serialized arrays for SM v1.4.14+)...  perhaps this
    // code should be removed a year (or more?) after 1.4.x
    // supports multiple select folder lists on the options page
    // (but not the unserialize() calls)
    //
    if (empty($sb_suppress_spam_button_folder))
       $sb_suppress_spam_button_folder = array();
-   else if (check_sm_version(1, 5, 2) && !is_array($sb_suppress_spam_button_folder))
-      $sb_suppress_spam_button_folder = unserialize($sb_suppress_spam_button_folder);
+   else if (check_sm_version(1, 4, 14) && !is_array($sb_suppress_spam_button_folder))
+      $sb_suppress_spam_button_folder = @unserialize($sb_suppress_spam_button_folder);
    if (!is_array($sb_suppress_spam_button_folder))
       $sb_suppress_spam_button_folder = array($sb_suppress_spam_button_folder);
 
    if (empty($sb_suppress_not_spam_button_folder))
       $sb_suppress_not_spam_button_folder = array();
-   else if (check_sm_version(1, 5, 2) && !is_array($sb_suppress_not_spam_button_folder))
-      $sb_suppress_not_spam_button_folder = unserialize($sb_suppress_not_spam_button_folder);
+   else if (check_sm_version(1, 4, 14) && !is_array($sb_suppress_not_spam_button_folder))
+      $sb_suppress_not_spam_button_folder = @unserialize($sb_suppress_not_spam_button_folder);
    if (!is_array($sb_suppress_not_spam_button_folder))
       $sb_suppress_not_spam_button_folder = array($sb_suppress_not_spam_button_folder);
 
    if (empty($sb_show_spam_button_folder))
       $sb_show_spam_button_folder = array();
-   else if (check_sm_version(1, 5, 2) && !is_array($sb_show_spam_button_folder))
-      $sb_show_spam_button_folder = unserialize($sb_show_spam_button_folder);
+   else if (check_sm_version(1, 4, 14) && !is_array($sb_show_spam_button_folder))
+      $sb_show_spam_button_folder = @unserialize($sb_show_spam_button_folder);
    if (!is_array($sb_show_spam_button_folder))
       $sb_show_spam_button_folder = array($sb_show_spam_button_folder);
 
    if (empty($sb_show_not_spam_button_folder))
       $sb_show_not_spam_button_folder = array();
-   else if (check_sm_version(1, 5, 2) && !is_array($sb_show_not_spam_button_folder))
-      $sb_show_not_spam_button_folder = unserialize($sb_show_not_spam_button_folder);
+   else if (check_sm_version(1, 4, 14) && !is_array($sb_show_not_spam_button_folder))
+      $sb_show_not_spam_button_folder = @unserialize($sb_show_not_spam_button_folder);
    if (!is_array($sb_show_not_spam_button_folder))
       $sb_show_not_spam_button_folder = array($sb_show_not_spam_button_folder);
 
@@ -133,20 +137,25 @@ function get_spam_buttons(&$buttons,
 
 
    // 1) is button turned on in the first place?
-   // 2)  a) if in (one of) the ham folder(s), show spam button
-   //     b) otherwise, if not using the ham folder (list), then
-   //        if in (one of) the spam folder(s), don't show spam button
-   // 3) if current message is tagged as spam, don't show spam button
-   // 4) can't show button if caller doesn't want it shown
+   // 2) either A or B below:
+   //     A) if current message is tagged as spam, don't show spam button
+   //     B) if not using message tag settings or not in message view, then:
+   //        a) if in (one of) the ham folder(s) ($sb_show_spam_button_folder
+   //           defines a list of ham folders to show the spam button in),
+   //           show spam button
+   //        b) otherwise, if not using the ham folder (list), then
+   //           if in (one of) the spam folder(s), don't show spam button
+   // 3) can't show button if caller doesn't want it shown
    //    when report method is move-to-folder and we're 
    //    looking at a message that is an attachment to another
    // 
-   if ($show_is_spam_button
-    && (in_array($mailbox, $sb_show_spam_button_folder)
-     || (empty($sb_show_spam_button_folder) 
-      && !in_array($mailbox, $sb_suppress_spam_button_folder)))
-    && !current_message_is_tagged(TRUE)
-    && (!$dont_show_if_report_by_move_to_folder
+   if ($show_is_spam_button                                              // 1
+    && (current_message_is_tagged(TRUE) === FALSE                        // 2A
+     || (current_message_is_tagged(TRUE) === 0                           // 2B
+      && (in_array($mailbox, $sb_show_spam_button_folder)                // 2Ba
+       || (empty($sb_show_spam_button_folder)                            // 2Bb
+        && !in_array($mailbox, $sb_suppress_spam_button_folder)))))
+    && (!$dont_show_if_report_by_move_to_folder                          // 3
      || !($sb_report_spam_by_move_to_folder && !empty($passed_ent_id))))
    {
       if (check_sm_version(1, 5, 2))
@@ -159,23 +168,28 @@ function get_spam_buttons(&$buttons,
 
 
    // 1) is button turned on in the first place?
-   // 2)  a) if in (one of) the spam folder(s), show ham button
-   //     b) otherwise, if not using the spam folder (list), then
-   //        if in (one of) the ham folder(s), don't show ham button
-   // 3) if current message is tagged as ham, don't show ham button
-   // 4) can't show button if caller doesn't want it shown
+   // 2) either A or B below:
+   //     A) if current message is tagged as ham, don't show ham button
+   //     B) if not using message tag settings or not in message view, then:
+   //        a) if in (one of) the spam folder(s) ($sb_show_not_spam_button_folder
+   //           defines a list of spam folders to show the ham button in),
+   //           show ham button
+   //        b) otherwise, if not using the spam folder (list), then
+   //           if in (one of) the ham folder(s), don't show ham button
+   // 3) can't show button if caller doesn't want it shown
    //    when report method is move-to-folder and we're 
    //    looking at a message that is an attachment to another
    // 
-   if ($show_not_spam_button
-    && (in_array($mailbox, $sb_show_not_spam_button_folder)
-     || (empty($sb_show_not_spam_button_folder) 
-      && !in_array($mailbox, $sb_suppress_not_spam_button_folder)))
-    && !current_message_is_tagged(FALSE)
-    && (!$dont_show_if_report_by_move_to_folder
+   if ($show_not_spam_button                                                 // 1
+    && (current_message_is_tagged(FALSE) === FALSE                           // 2A
+     || (current_message_is_tagged(FALSE) === 0                              // 2B
+      && (in_array($mailbox, $sb_show_not_spam_button_folder)                // 2Ba
+       || (empty($sb_show_not_spam_button_folder)                            // 2Bb
+        && !in_array($mailbox, $sb_suppress_not_spam_button_folder)))))
+    && (!$dont_show_if_report_by_move_to_folder                              // 3
      || !($sb_report_not_spam_by_move_to_folder && !empty($passed_ent_id))))
    {
-      if (check_sm_version(1, 5, 1))
+      if (check_sm_version(1, 5, 2))
          $buttons['notSpam'] = array('value' => _($not_spam_button_text), 'type' => 'submit');
       else if (check_sm_version(1, 5, 1))
          $buttons[1]['notSpam'] = array(0 => _($not_spam_button_text), 1 => 'submit');
@@ -183,6 +197,28 @@ function get_spam_buttons(&$buttons,
          $ret .= '<input type="submit" name="notSpam" value="' . _($not_spam_button_text) . "\" />\n";
    }
 
+
+   // add any extra buttons
+   //
+   foreach ($extra_buttons as $button => $button_info)
+   {
+
+      // build button
+      //
+      if (sb_show_custom_button_or_link($button, $button_info))
+      {
+         $button_name = preg_replace('/[^a-zA-Z0-9]/', '_', $button);
+         if (check_sm_version(1, 5, 2))
+            $buttons[$button_name] = array('value' => _($button), 'type' => 'submit');
+         else if (check_sm_version(1, 5, 1))
+            $buttons[1][$button_name] = array(0 => _($button), 1 => 'submit');
+         else
+            $ret .= '<input type="submit" name="' . $button_name . '" value="' . _($button) . "\" />\n";
+      }
+
+   }
+
+
    sq_change_text_domain('squirrelmail');
 
    return $ret;
@@ -210,7 +246,7 @@ function sb_read_message_links_do(&$links)
           $sb_suppress_not_spam_button_folder_allow_override,
           $sb_show_spam_button_folder,
           $sb_show_spam_button_folder_allow_override,
-          $sb_show_not_spam_button_folder,
+          $sb_show_not_spam_button_folder, $extra_buttons,
           $sb_show_not_spam_button_folder_allow_override;
 
    spam_buttons_init();
@@ -247,36 +283,36 @@ function sb_read_message_links_do(&$links)
    // these options used to be a strings; these lines convert
    // values that may have already been stored in user prefs
    // as strings into proper array format (also unpacks
-   // serialized arrays for SM v1.5.2+)...  perhaps this
+   // serialized arrays for SM v1.4.14+)...  perhaps this
    // code should be removed a year (or more?) after 1.4.x
    // supports multiple select folder lists on the options page
    // (but not the unserialize() calls)
    //
    if (empty($sb_suppress_spam_button_folder))
       $sb_suppress_spam_button_folder = array();
-   else if (check_sm_version(1, 5, 2) && !is_array($sb_suppress_spam_button_folder))
-      $sb_suppress_spam_button_folder = unserialize($sb_suppress_spam_button_folder);
+   else if (check_sm_version(1, 4, 14) && !is_array($sb_suppress_spam_button_folder))
+      $sb_suppress_spam_button_folder = @unserialize($sb_suppress_spam_button_folder);
    if (!is_array($sb_suppress_spam_button_folder))
       $sb_suppress_spam_button_folder = array($sb_suppress_spam_button_folder);
 
    if (empty($sb_suppress_not_spam_button_folder))
       $sb_suppress_not_spam_button_folder = array();
-   else if (check_sm_version(1, 5, 2) && !is_array($sb_suppress_not_spam_button_folder))
-      $sb_suppress_not_spam_button_folder = unserialize($sb_suppress_not_spam_button_folder);
+   else if (check_sm_version(1, 4, 14) && !is_array($sb_suppress_not_spam_button_folder))
+      $sb_suppress_not_spam_button_folder = @unserialize($sb_suppress_not_spam_button_folder);
    if (!is_array($sb_suppress_not_spam_button_folder))
       $sb_suppress_not_spam_button_folder = array($sb_suppress_not_spam_button_folder);
 
    if (empty($sb_show_spam_button_folder))
       $sb_show_spam_button_folder = array();
-   else if (check_sm_version(1, 5, 2) && !is_array($sb_show_spam_button_folder))
-      $sb_show_spam_button_folder = unserialize($sb_show_spam_button_folder);
+   else if (check_sm_version(1, 4, 14) && !is_array($sb_show_spam_button_folder))
+      $sb_show_spam_button_folder = @unserialize($sb_show_spam_button_folder);
    if (!is_array($sb_show_spam_button_folder))
       $sb_show_spam_button_folder = array($sb_show_spam_button_folder);
 
    if (empty($sb_show_not_spam_button_folder))
       $sb_show_not_spam_button_folder = array();
-   else if (check_sm_version(1, 5, 2) && !is_array($sb_show_not_spam_button_folder))
-      $sb_show_not_spam_button_folder = unserialize($sb_show_not_spam_button_folder);
+   else if (check_sm_version(1, 4, 14) && !is_array($sb_show_not_spam_button_folder))
+      $sb_show_not_spam_button_folder = @unserialize($sb_show_not_spam_button_folder);
    if (!is_array($sb_show_not_spam_button_folder))
       $sb_show_not_spam_button_folder = array($sb_show_not_spam_button_folder);
 
@@ -294,31 +330,51 @@ function sb_read_message_links_do(&$links)
    sq_change_text_domain('spam_buttons');
 
 //TODO: create tooltip with more verbose explanation 
-   // do not show buttons if the report-by-move-to-folder method
-   // has been configured and $passed_ent_id is non-zero (can't
-   // move an attachment)... and of course, not if in (one of) the
-   // spam/ham folder(s) (unless in (one of) the ham/spam folder(s))
-   // or if message is tagged as spam/ham
+   // 1) is button turned on in the first place?
+   // 2) either A or B below: 
+   //     A) if current message is tagged as spam, don't show spam button
+   //     B) if not using message tag settings, then:
+   //        a) if in (one of) the ham folder(s) ($sb_show_spam_button_folder
+   //           defines a list of ham folders to show the spam button in),
+   //           show spam button
+   //        b) otherwise, if not using the ham folder (list), then
+   //           if in (one of) the spam folder(s), don't show spam button
+   // 3) can't show button if report method is move-to-folder and we're
+   //    looking at a message that is an attachment to another
    //
-   if ($show_is_spam_button 
-    && (in_array($mailbox, $sb_show_spam_button_folder)
-     || (empty($sb_show_spam_button_folder) 
-      && !in_array($mailbox, $sb_suppress_spam_button_folder)))
-    && !current_message_is_tagged(TRUE)
-    && !($sb_report_spam_by_move_to_folder && !empty($passed_ent_id)))
+   if ($show_is_spam_button                                              // 1
+    && (current_message_is_tagged(TRUE) === FALSE                        // 2A
+     || (current_message_is_tagged(TRUE) === 0                           // 2B
+      && (in_array($mailbox, $sb_show_spam_button_folder)                // 2Ba
+       || (empty($sb_show_spam_button_folder)                            // 2Bb
+        && !in_array($mailbox, $sb_suppress_spam_button_folder)))))
+    && !($sb_report_spam_by_move_to_folder && !empty($passed_ent_id)))   // 3
       $spam_links[] = array('URL' => sqm_baseuri() 
          . 'src/read_body.php?isSpam=yslnk&mailbox=' 
          . urlencode($mailbox) . '&passed_id=' . $passed_id . '&view_as_html='
          . $view_as_html . '&startMessage=' . $startMessage . '&passed_ent_id=' 
          . $passed_ent_id . '&account=' . $account, 
          'Text' => _($spam_button_text));
-   
-   if ($show_not_spam_button
-    && (in_array($mailbox, $sb_show_not_spam_button_folder)
-     || (empty($sb_show_not_spam_button_folder) 
-      && !in_array($mailbox, $sb_suppress_not_spam_button_folder)))
-    && !current_message_is_tagged(FALSE)
-    && !($sb_report_not_spam_by_move_to_folder && !empty($passed_ent_id)))
+
+   // 1) is button turned on in the first place?
+   // 2) either A or B below: 
+   //     A) if current message is tagged as ham, don't show ham button
+   //     B) if not using message tag settings, then:
+   //        a) if in (one of) the spam folder(s) ($sb_show_not_spam_button_folder
+   //           defines a list of spam folders to show the ham button in),
+   //           show ham button
+   //        b) otherwise, if not using the spam folder (list), then
+   //           if in (one of) the ham folder(s), don't show ham button
+   // 3) can't show button if report method is move-to-folder and we're
+   //    looking at a message that is an attachment to another
+   //
+   if ($show_not_spam_button                                                 // 1
+    && (current_message_is_tagged(FALSE) === FALSE                           // 2A
+     || (current_message_is_tagged(FALSE) === 0                              // 2B
+      && (in_array($mailbox, $sb_show_not_spam_button_folder)                // 2Ba
+       || (empty($sb_show_not_spam_button_folder)                            // 2Bb
+        && !in_array($mailbox, $sb_suppress_not_spam_button_folder)))))
+    && !($sb_report_not_spam_by_move_to_folder && !empty($passed_ent_id)))   // 3
       $spam_links[] = array('URL' => sqm_baseuri() 
          . 'src/read_body.php?notSpam=yslnk&mailbox='
          . urlencode($mailbox) . '&passed_id=' . $passed_id . '&view_as_html='
@@ -335,6 +391,33 @@ function sb_read_message_links_do(&$links)
          echo ' | <a href="' . $link['URL'] . '">' . $link['Text'] . '</a>';
    }
 
+
+   // add any extra links
+   //
+   foreach ($extra_buttons as $button => $button_info)
+   {
+
+      // build button
+      //
+      if (sb_show_custom_button_or_link($button, $button_info))
+      {
+         $button_name = preg_replace('/[^a-zA-Z0-9]/', '_', $button);
+         $button_link = sqm_baseuri() . 'src/read_body.php?' . $button_name
+                      . '=yslnk&mailbox=' . urlencode($mailbox) . '&passed_id='
+                      . $passed_id . '&view_as_html=' . $view_as_html
+                      . '&startMessage=' . $startMessage . '&passed_ent_id='
+                      . $passed_ent_id . '&account=' . $account;
+         if (check_sm_version(1, 5, 2))
+            $links[] = array('URL'  => $button_link,
+                             'Text' => $button);
+
+         else
+            echo ' | <a href="' . $button_link . '">' . $button . '</a>';
+      }
+
+   }
+
+
    sq_change_text_domain('squirrelmail');
 
 }
@@ -469,12 +552,18 @@ function sb_read_message_buttons_do($args)
   *
   * If the configuration items related to this functionality are
   * not set, or we are not on a message view screen, this always 
-  * returns FALSE.
+  * returns 0 (see WARNING below).
   *
   * @param boolean $spam When TRUE, check if the message is spam,
   *                      when FALSE, check if the message is ham.
   *
-  * @return boolean TRUE if the message has been tagged as inquired
+  * @return mixed TRUE if the message has been tagged as inquired
+  *               FALSE if the message has not been tagged as such
+  *               0 if the needed configuration settings are not
+  *               set up in the configuration file or the current
+  *               page request is not the message view page.  
+  *               WARNING: callers need to carefully check for 
+  *               either 0 or FALSE (using === or !==).
   *
   */
 function current_message_is_tagged($spam)
@@ -482,13 +571,13 @@ function current_message_is_tagged($spam)
 
    if (defined('PAGE_NAME'))
    {
-      if (PAGE_NAME != 'read_body') return FALSE;
+      if (PAGE_NAME != 'read_body') return 0;
    }
    else
    {
       global $PHP_SELF;
       if (strpos($PHP_SELF, '/src/read_body') === FALSE) 
-         return FALSE;
+         return 0;
    }
 
    global $sb_spam_header_name, $sb_not_spam_header_name,
@@ -499,122 +588,232 @@ function current_message_is_tagged($spam)
    sqgetGlobalVar('passed_id',     $passed_id,     SQ_FORM);
    sqgetGlobalVar('passed_ent_id', $passed_ent_id, SQ_FORM);
 
-   if ($spam && !empty($sb_spam_header_name) 
-    && !empty($sb_spam_header_value))
+   if ($spam)
    {
 
-      $headers = get_message_headers($passed_id, $passed_ent_id);
+      if (empty($sb_spam_header_name) || empty($sb_spam_header_value))
+         return 0;
+
+      $header = sb_get_message_header($passed_id, $passed_ent_id, $sb_spam_header_name);
+      if (preg_match($sb_spam_header_value, trim($header[1])))
+         return TRUE;
+
+/* ......old code, less efficient; left for posterity............................
+      $headers = sb_get_message_headers($passed_id, $passed_ent_id);
       foreach ($headers as $header)
          if (strtolower($header[0]) == strtolower($sb_spam_header_name . ':')
           && preg_match($sb_spam_header_value, trim($header[1])))
             return TRUE;
+............................................................................. */
+
+      return FALSE;
 
    }
-   else if (!$spam && !empty($sb_not_spam_header_name) 
-    && !empty($sb_not_spam_header_value))
+   else 
    {
 
-      $headers = get_message_headers($passed_id, $passed_ent_id);
+      if (empty($sb_not_spam_header_name) || empty($sb_not_spam_header_value))
+         return 0;
+
+      $header = sb_get_message_header($passed_id, $passed_ent_id, $sb_not_spam_header_name);
+      if (preg_match($sb_not_spam_header_value, trim($header[1])))
+         return TRUE;
+
+/* ......old code, less efficient; left for posterity............................
+      $headers = sb_get_message_headers($passed_id, $passed_ent_id);
       foreach ($headers as $header)
          if (strtolower($header[0]) == strtolower($sb_not_spam_header_name . ':')
           && preg_match($sb_not_spam_header_value, trim($header[1])))
             return TRUE;
+............................................................................. */
+
+      return FALSE;
 
    }
 
-   return FALSE;
-
 }
 
 
 
 /**
-  * Retrieve All Message Headers
+  * Determines if a custom/extra button or link should be shown.
   *
-  * When called more than once in the same page request against the same
-  * message, a cached set of headers is returned for efficiency.
+  * @param string $button_name The name of the button or link
+  * @param array  $button_info The array of button callbacks and
+  *                            associated information from the
+  *                            configuration file
   *
-  * @param int    $passed_id     UID of the desired message
-  * @param string $passed_ent_id Entity ID of the desired message attachment
-  *
-  * @return array An array of all message headers
+  * @return boolean TRUE if the button should be shown, or FALSE
+  *                 if it should not.
   *
   */
-function get_message_headers($passed_id, $passed_ent_id)
+function sb_show_custom_button_or_link($button_name, $button_info)
 {
 
-   static $parsed_headers = array();
-   static $id = NULL;
-   static $ent_id = NULL;
-
-   if (!empty($parsed_headers) && $passed_id == $id && $passed_ent_id == $ent_id)
-      return $parsed_headers;
-
-   $id = $passed_id;
-   $ent_id = $passed_ent_id;
-
-   global $imapConnection, $username, $key, $uid_support,
-          $mbx_response, $imapServerAddress, $imapPort, $mailbox; 
-
-   if (check_sm_version(1, 5, 2)) { $key = FALSE; $uid_support = TRUE; }
-
-   if (!is_resource($imapConnection))
-      $imapConnection = sqimap_login($username, $key, $imapServerAddress, $imapPort, 0);
-   if (empty($mbx_response))
-      $mbx_response = sqimap_mailbox_select($imapConnection, $mailbox);
-
-   if (empty($ent_id))
-      $headers = sqimap_run_command($imapConnection, "FETCH $id BODY[HEADER]", true, $response, $message, $uid_support);
-   else
-      $headers = sqimap_run_command($imapConnection, "FETCH $id BODY[$ent_id.HEADER]", true, $response, $message, $uid_support);
+   global $username;
+   $callback = '';
+   $action = '';
+   $passed_id = '';
+   $passed_ent_id = '';
+   $from = '';
+   $error = '';
+   $show_it = FALSE;
 
 
-   // based on src/view_header.php...
+   // this function is found in the Compatibility plugin v2.0.5+
    //
-   $first = $second = array();
-   $cnum = 0;
+   $hook_name = get_current_hook_name();
 
-   for ($i=1; $i < count($headers); $i++) 
+
+   sq_change_text_domain('spam_buttons');
+
+
+   // in this case, SquirrelMail is building the buttons at the
+   // top of the message list screen
+   //
+   if ($hook_name == 'mailbox_display_buttons'  // SM 1.4.x
+    || $hook_name == 'message_list_controls')  // SM 1.5.x
    {
-      $line = $headers[$i];
-      switch (true) 
+      $action = 'MESSAGE_LIST_BUTTON';
+      if (!isset($button_info[0]))
+         $error = sprintf(_("Spam Buttons plugin is not configured correctly!  Check custom button configuration array for \"%s\""), $button_name);
+      else
+         $callback = $button_info[0];
+   }
+
+
+   // in this case, SquirrelMail is building the "Options" links
+   // on the message view screen
+   //
+   else if ($hook_name == 'read_body_header_right')
+   {
+
+      $action = 'MESSAGE_VIEW_LINK';
+      if (!isset($button_info[1]))
+         $error = sprintf(_("Spam Buttons plugin is not configured correctly!  Check custom button configuration array for \"%s\""), $button_name);
+      else
+         $callback = $button_info[1];
+
+
+      // identify the message being viewed in case they are needed
+      //
+      sqgetGlobalVar('passed_id',     $passed_id,     SQ_FORM);
+      sqgetGlobalVar('passed_ent_id', $passed_ent_id, SQ_FORM);
+
+
+      // this retrieves the message's From header in the format
+      // array(0 => 'From:', 1 => '"Jose" <jose@example.org>')
+      //
+      $from = sb_get_message_header($passed_id, $passed_ent_id, 'From');
+
+
+      // this parses out just the email address portion of the From header
+      //
+      if (function_exists('parseRFC822Address'))
       {
-         case (eregi("^&gt;", $line)):
-            $second[$i] = $line;
-            $first[$i] = '&nbsp;';
-            $cnum++;
-            break;
-         case (eregi("^[ |\t]", $line)):
-            $second[$i] = $line;
-            $first[$i] = '';
-            break;
-         case (eregi("^([^:]+):(.+)", $line, $regs)):
-            $first[$i] = $regs[1] . ':';
-            $second[$i] = $regs[2];
-            $cnum++;
-            break;
-         default:
-            $second[$i] = trim($line);
-            $first[$i] = '';
-            break;
+         $from = parseRFC822Address($from[1], 1);
+         $from = $from[0][2] . '@' . $from[0][3];
+      }
+      else
+      {
+         $from = parseAddress($from[1], 1);
+         $from = $from[0][0];
+      }
+
+   }
+
+
+   // in this case, SquirrelMail is building the buttons in the
+   // message view screen's button row (SquirrelMail 1.5.2+ only)
+   //
+   else if ($hook_name == 'template_construct_read_menubar_buttons.tpl')
+   {
+// NOTE: at some point, 1.5.x is likely to change the API for the buttons on the read message screen, then this code will have to change to suit
+
+      $action = 'MESSAGE_VIEW_BUTTON';
+      if (!isset($button_info[2]))
+         $error = sprintf(_("Spam Buttons plugin is not configured correctly!  Check custom button configuration array for \"%s\""), $button_name);
+      else
+         $callback = $button_info[2];
+
+
+      // these values identify the message being viewed
+      // in case you need them
+      //
+      sqgetGlobalVar('passed_id',     $passed_id,     SQ_FORM);
+      sqgetGlobalVar('passed_ent_id', $passed_ent_id, SQ_FORM);
+
+
+      // this retrieves the message's From header in the format
+      // array(0 => 'From:', 1 => '"Jose" <jose@example.org>')
+      //
+      $from = sb_get_message_header($passed_id, $passed_ent_id, 'From');
+
+
+      // this parses out just the email address portion of the From header
+      //
+      if (function_exists('parseRFC822Address'))
+      {
+         $from = parseRFC822Address($from[1], 1);
+         $from = $from[0][2] . '@' . $from[0][3];
+      }
+      else
+      {
+         $from = parseAddress($from[1], 1);
+         $from = $from[0][0];
+      }
+
+   }
+
+
+   // is the button always to be shown, or never to be shown?
+   //
+   if (empty($error))
+   {
+      if ($callback === 1 || $callback === '1')
+      {
+         sq_change_text_domain('squirrelmail');
+         return TRUE;
+      }
+      if ($callback === 0 || $callback === '0')
+      {
+         sq_change_text_domain('squirrelmail');
+         return FALSE;
       }
    }
 
-   for ($i=0; $i < count($second); $i = $j) 
+
+   // otherwise, execute callback to figure it out...
+   //
+   if (empty($callback))
+      $error = sprintf(_("Spam Buttons plugin is not configured correctly!  Check custom button configuration array for \"%s\""), $button_name);
+   else if (!function_exists($callback))
+      $error = sprintf(_("Function \"%s\" not found in Spam Buttons plugin"), $callback);
+   else
+      $show_it = $callback($action, $username, $from, $passed_id, $passed_ent_id);
+
+
+   // error?
+   //
+   if (!empty($error))
    {
-      $f = (isset($first[$i]) ? $first[$i] : '');
-      $s = (isset($second[$i]) ? $second[$i] : '');
-      $j = $i + 1;
-      while (($first[$j] == '') && ($j < count($first))) 
+      global $color;
+      sq_change_text_domain('squirrelmail');
+      $ret = plain_error_message($error, $color);
+      if (check_sm_version (1, 5, 2))
       {
-         $s .= ' ' . $second[$j];
-         $j++;
+         echo $ret;
+         global $oTemplate;
+         $oTemplate->display('footer.tpl');
       }
-      if ($f) 
-         $parsed_headers[] = array($f,$s);
+      exit;
    }
 
-   return $parsed_headers;
+
+   // return value from the callback
+   //
+   sq_change_text_domain('squirrelmail');
+   return $show_it;
 
 }
 
