@@ -306,12 +306,12 @@ function getMessage_RFC822_Attachment($message, $composeMessage, $passed_id,
     $hashed_attachment_dir = getHashedDir($username, $attachment_dir);
     if (!$passed_ent_id) {
         $body_a = sqimap_run_command($imapConnection,
-                'FETCH '.$passed_id.' RFC822',
+                'FETCH '.$passed_id.' BODY.PEEK[]',
                 TRUE, $response, $readmessage,
                 $uid_support);
     } else {
         $body_a = sqimap_run_command($imapConnection,
-                'FETCH '.$passed_id.' BODY['.$passed_ent_id.']',
+                'FETCH '.$passed_id.' BODY.PEEK['.$passed_ent_id.']',
                 TRUE, $response, $readmessage, $uid_support);
         $message = $message->parent;
     }
@@ -541,6 +541,9 @@ function deliverMessage($composeMessage, $draft=false) {
         $imap_stream = sqimap_login($username, $key, $imapServerAddress, $imapPort, 0);
 
         /* Move to sent code */
+/// CHANGE FOR SPAM_BUTTONS PLUGIN (added next 2 lines)
+global $sb_keep_copy_in_sent;
+if ($sb_keep_copy_in_sent) {
         if (isset($default_move_to_sent) && ($default_move_to_sent != 0)) {
             $svr_allow_sent = true;
         } else {
@@ -568,6 +571,8 @@ function deliverMessage($composeMessage, $draft=false) {
             sqimap_append_done ($imap_stream, $sent_folder);
             unset ($imap_deliver);
         }
+/// CHANGE FOR SPAM_BUTTONS PLUGIN (added next line)
+}
         global $passed_id, $mailbox, $action;
         $composeMessage->purgeAttachments();
         if ($action == 'reply' || $action == 'reply_all') {
